# setup bashrc file: to set the $PATH and $LD_LIBRARY_PATH variable in a
# linux computer.
# the PVSS_SYSTEM_ROOT/bin, the INSTALLED_COMPONENTS/bin and 
# the PROJECT_ROOT/bin are added to the $PATH and $LD_LIBRARY_PATH 
# environment variable
#
# creation date: 11 March 2003
# update date: 29 February 2008 -> add java path, PVSS 3.6SP2
# author: Herve Milcent

# You should edit the following definitions to reflect your own setup.

# Where is the pvss system?
export PVSS_SYSTEM_ROOT="/opt/pvss/pvss2_v3.6"
#compile api, all libraries ...
export API_ROOT=$PVSS_SYSTEM_ROOT/api
export PLATFORM=`${API_ROOT}/platform`
export LD_LIBRARY_PATH=$PVSS_SYSTEM_ROOT/bin:/home/unicryo/PVSS_projects/UNICOS_PVSS36/PERF36/installed_components/bin

# Where is your project?
export PROJECT_ROOT="/home/unicryo/PVSS_projects/UNICOS_PVSS36/PERF36/PERF36"

# Where is the framework?
export INSTALLED_COMPONENTS="/home/unicryo/PVSS_projects/UNICOS_PVSS36/PERF36/installed_components"

#-----------------------------------------------------------------------
#
# You should not need to edit below this line!
#

# Tell the location of the PVSS package. needed for PVSS II v3:
export PVSS_II_HOME=$PVSS_SYSTEM_ROOT

# Tell PVSS how to find the project's config file:
export PVSS_II="$PROJECT_ROOT/config/config"

echo "PVSS_SYSTEM_ROOT=$PVSS_SYSTEM_ROOT"
echo "PVSS_II=$PVSS_II"
echo "PVSS_II_HOME=$PVSS_II_HOME"
echo "PROJECT_ROOT = $PROJECT_ROOT"
echo "PLATFORM = $PLATFORM"

# Make the PVSS tools available in PATH.
# The fancy bit here means that this script can be re-run without making
# the PATH infinitely long.
# Rid any previous PVSS bin directory from the path
if [ ${PVSS_SYSTEM_ROOT}null != 'null' ] 
then 
  echo $PATH | grep $PVSS_SYSTEM_ROOT/bin 1>/dev/null 2>&1
  if [ $? -eq 0 ] 
  then
    PATH=`echo $PATH | sed sI:$PVSS_SYSTEM_ROOT/binII`
  fi
fi

# Add the PVSS bin directory to the path
echo $PATH | grep $PVSS_SYSTEM_ROOT 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then 
  PATH=`echo $PATH | sed 's/\(.*\):$/\1/'`:$PVSS_SYSTEM_ROOT/bin
fi 

# add java path
echo $PATH | grep /usr/java/jdk1.6.0/bin 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then
   PATH=`echo $PATH | sed 's/\(.*\):$/\1/'`:/usr/java/jdk1.6.0/bin
fi

#echo "PVSS_SYSTEM_ROOT PATH=$PATH"
#echo "PVSS_SYSTEM_ROOT  LD_LIBRARY_PATH(1) = $LD_LIBRARY_PATH"

# Setup the library path defintion too.
# The fancy bit here means that this script can be re-run without making
# the LD_LIBRARY_PATH infinitely long.
# Rid any previous PVSS library directory from the path
if [ ${PVSS_SYSTEM_ROOT}null != 'null' ] 
then 
  echo $LD_LIBRARY_PATH | grep $PVSS_SYSTEM_ROOT/bin 1>/dev/null 2>&1
  if [ $? -eq 0 ] 
  then
    LD_LIBRARY_PATH=`echo $LD_LIBRARY_PATH | sed sI:$PVSS_SYSTEM_ROOT/binII`
  fi
fi

#echo "LD_LIBRARY_PATH(2) = $LD_LIBRARY_PATH"


# Add the PVSS bin directory to the LD_LIBRARY_PATH
echo $LD_LIBRARY_PATH | grep $PVSS_SYSTEM_ROOT 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then 
#  echo "then was true"
  LD_LIBRARY_PATH=`echo $LD_LIBRARY_PATH | sed 's/\(.*\):$/\1/'`:$PVSS_SYSTEM_ROOT/bin
fi 
# Remove any leading colon:
#echo "LD_LIBRARY_PATH(3) = $LD_LIBRARY_PATH"
export LD_LIBRARY_PATH=`echo ${LD_LIBRARY_PATH} | sed 's/^://'`
#echo "LD_LIBRARY_PATH(4) = $LD_LIBRARY_PATH"

#export LD_LIBRARY_PATH=${PVSS_SYSTEM_ROOT}/bin
#echo "Final LD_LIBRARY_PATH=$LD_LIBRARY_PATH"
#echo "PVSS_SYSTEM_ROOT PATH=$PATH"
#echo "PVSS_SYSTEM_ROOT  LD_LIBRARY_PATH(1) = $LD_LIBRARY_PATH"

# Add the PROJECT_ROOT bin directory to the path
echo $PATH | grep $PROJECT_ROOT 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then 
  PATH=`echo $PATH | sed 's/\(.*\):$/\1/'`:$PROJECT_ROOT/bin
fi 
#echo "PATH=$PATH"

# Add the PROJECT_ROOT bin directory to the LD_LIBRARY_PATH
echo $LD_LIBRARY_PATH | grep $PROJECT_ROOT 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then 
  #echo "then was true"
  LD_LIBRARY_PATH=`echo $LD_LIBRARY_PATH | sed 's/\(.*\):$/\1/'`:$PROJECT_ROOT/bin
fi 
#echo "LD_LIBRARY_PATH(1) = $LD_LIBRARY_PATH"
#echo "PROJECT_ROOT PATH=$PATH"
#echo "PROJECT_ROOT  LD_LIBRARY_PATH(1) = $LD_LIBRARY_PATH"

# Add the INSTALLED_COMPONENTS bin directory to the path
echo $PATH | grep $INSTALLED_COMPONENTS 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then 
  PATH=`echo $PATH | sed 's/\(.*\):$/\1/'`:$INSTALLED_COMPONENTS/bin
fi 
#echo "PATH=$PATH"

# Add the INSTALLED_COMPONENTS bin directory to the LD_LIBRARY_PATH
echo $LD_LIBRARY_PATH | grep $INSTALLED_COMPONENTS 1>/dev/null 2>&1
if [ $? -eq 1 ] 
then 
  #echo "then was true"
  LD_LIBRARY_PATH=`echo $LD_LIBRARY_PATH | sed 's/\(.*\):$/\1/'`:$INSTALLED_COMPONENTS/bin
fi 
#echo "LD_LIBRARY_PATH(1) = $LD_LIBRARY_PATH"
#echo "INSTALLED_COMPONENTS PATH=$PATH"
#echo "INSTALLED_COMPONENTS  LD_LIBRARY_PATH(1) = $LD_LIBRARY_PATH"

